@@ -28,10 +28,6 @@ G_BEGIN_DECLS
 struct _GODocControl {
 	GObject		 base;
 
-	struct {
-		gpointer dummy;
-	} state[GO_DOC_CONTROL_STATE_MAX];
-
 	GODoc		*doc;
 };
 
