@@ -396,9 +396,11 @@ save_image_cb (gpointer key, gpointer img_, gpointer user)
 void
 go_doc_write (GODoc *doc, GsfXMLOut *output)
 {
-	gsf_xml_out_start_element (output, "GODoc");
-	g_hash_table_foreach (doc->imagebuf, save_image_cb, output);
-	gsf_xml_out_end_element (output);
+	if (g_hash_table_size (doc->imagebuf) > 0) {
+		gsf_xml_out_start_element (output, "GODoc");
+		g_hash_table_foreach (doc->imagebuf, save_image_cb, output);
+		gsf_xml_out_end_element (output);
+	}
 	g_hash_table_destroy (doc->imagebuf);
 	doc->imagebuf = NULL;
 }
@@ -421,10 +423,10 @@ load_image (GsfXMLIn *xin, xmlChar const **attrs)
 	GODoc *doc = GO_DOC (xin->user_state);
 	GOImage *image;
 	xmlChar const **attr = 	attrs;
+	if (!*attr)
+		return;
 	while (*attr && strcmp (*attr, "name"))
 		attr += 2;
-	if (!attr)
-		return;
 	image = (GOImage *) g_hash_table_lookup (doc->imagebuf, attr[1]);
 	if (!image) /* this should not occur, but if it does, we might want to load the image? */
 		return;
