@@ -22,6 +22,7 @@
 
 #include <goffice/utils/go-glib-extras.h>
 #include <goffice/utils/go-libxml-extras.h>
+#include <goffice/utils/go-file.h>
 #include <libxml/parser.h>
 #include <libxml/parserInternals.h>
 #include <libxml/xmlmemory.h>
@@ -35,7 +36,9 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
+#endif
 #include <locale.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -55,7 +58,7 @@ static GHashTable *loader_services = NULL;
 static GType	   go_default_loader_type;
 
 static void plugin_get_loader_if_needed (GOPlugin *plugin, ErrorInfo **ret_error);
-static void go_plugin_read (GOPlugin *plugin, const gchar *dir_name, ErrorInfo **ret_error);
+static void go_plugin_read (GOPlugin *plugin, gchar const *dir_name, ErrorInfo **ret_error);
 static void go_plugin_load_base (GOPlugin *plugin, ErrorInfo **ret_error);
 
 /*
@@ -113,7 +116,7 @@ static GObjectClass *parent_class = NULL;
 static void plugin_dependency_free (gpointer data);
 
 static void
-go_plugin_message (gint level, const gchar *format, ...)
+go_plugin_message (gint level, gchar const *format, ...)
 {
 #ifdef PLUGIN_DEBUG
 	va_list args;
@@ -201,7 +204,7 @@ GSF_CLASS (GOPlugin, go_plugin, go_plugin_class_init, go_plugin_init,
            G_TYPE_OBJECT)
 
 static GOPlugin *
-go_plugin_new_from_xml (const gchar *dir_name, ErrorInfo **ret_error)
+go_plugin_new_from_xml (gchar const *dir_name, ErrorInfo **ret_error)
 {
 	GOPlugin *plugin;
 	ErrorInfo *error;
@@ -221,7 +224,7 @@ go_plugin_new_from_xml (const gchar *dir_name, ErrorInfo **ret_error)
 }
 
 static GOPlugin *
-go_plugin_new_with_id_and_dir_name_only (const gchar *id, const gchar *dir_name)
+go_plugin_new_with_id_and_dir_name_only (gchar const *id, gchar const *dir_name)
 {
 	GOPlugin *plugin = g_object_new (GO_PLUGIN_TYPE, NULL);
 	plugin->id = g_strdup (id);
@@ -247,7 +250,7 @@ static gboolean plugin_file_state_hash_changed;
 static GHashTable *plugin_file_state_dir_hash;
 
 static gchar *
-get_file_state_as_string (const gchar *file_name)
+get_file_state_as_string (gchar const *file_name)
 {
 	struct stat st;
 
@@ -269,7 +272,7 @@ plugin_file_state_as_string (PluginFileState *state)
 }
 
 static PluginFileState *
-plugin_file_state_from_string (const gchar *str)
+plugin_file_state_from_string (gchar const *str)
 {
 	PluginFileState *state;
 	gchar **strv;
@@ -361,12 +364,12 @@ go_plugin_read_full_info_if_needed (GOPlugin *plugin)
 
 /**
  * go_plugin_get_textdomain:
- * @plugin      : The plugin
+ * @plugin : #GOPlugin
  *
- * Returns plugin's textdomain for use with textdomain(3) and d*gettext(3)
- * functions.
- */
-const gchar *
+ * Returns: plugin's textdomain for use with textdomain(3) and d*gettext(3)
+ * 	functions.
+ **/
+gchar const *
 go_plugin_get_textdomain (GOPlugin *plugin)
 {
 	g_return_val_if_fail (IS_GO_PLUGIN (plugin), NULL);
@@ -380,10 +383,10 @@ go_plugin_get_textdomain (GOPlugin *plugin)
 
 /**
  * go_plugin_is_active:
- * @plugin      : The plugin
+ * @plugin : #GOPlugin
  *
- * Returns : TRUE if @plugin is active and FALSE otherwise.
- */
+ * Returns: TRUE if @plugin is active and FALSE otherwise.
+ **/
 gboolean
 go_plugin_is_active (GOPlugin *plugin)
 {
@@ -416,7 +419,7 @@ static GSF_CLASS (GOPluginTypeModule, go_plugin_type_module,
  * go_plugin_get_type_module :
  * @plugin : #GOPlugin
  *
- * Return the GTypeModule associated with the plugin creating it if necessary.
+ * Returns: the GTypeModule associated with the plugin creating it if necessary.
  **/
 GTypeModule *
 go_plugin_get_type_module (GOPlugin *plugin)
@@ -433,12 +436,12 @@ go_plugin_get_type_module (GOPlugin *plugin)
 
 /**
  * go_plugin_get_dir_name:
- * @plugin      : The plugin
+ * @plugin : #GOPlugin
  *
- * Returns the name of the directory in which @plugin is located.
- * Returned string is != NULL and stays valid during @plugin's lifetime.
- */
-const gchar *
+ * Returns: the name of the directory in which @plugin is located.
+ * 	Returned string is != NULL and stays valid during @plugin's lifetime.
+ **/
+gchar const *
 go_plugin_get_dir_name (GOPlugin *plugin)
 {
 	g_return_val_if_fail (IS_GO_PLUGIN (plugin), NULL);
@@ -448,13 +451,13 @@ go_plugin_get_dir_name (GOPlugin *plugin)
 
 /**
  * go_plugin_get_id:
- * @plugin      : The plugin
+ * @plugin : #GOPlugin
  *
- * Returns the ID of @plugin (unique string used for idenfification of
- * plugin).
- * Returned string is != NULL and stays valid during @plugin's lifetime.
- */
-const gchar *
+ * Returns: the ID of @plugin (unique string used for idenfification of
+ * 	plugin).  Returned string is != NULL and stays valid during @plugin's
+ * 	lifetime.
+ **/
+gchar const *
 go_plugin_get_id (GOPlugin *plugin)
 {
 	g_return_val_if_fail (IS_GO_PLUGIN (plugin), NULL);
@@ -464,13 +467,13 @@ go_plugin_get_id (GOPlugin *plugin)
 
 /**
  * go_plugin_get_name:
- * @plugin      : The plugin
+ * @plugin : #GOPlugin
  *
- * Returns textual name of @plugin. If the real name is not available
- * for some reason, automatically generated string will be returned.
- * Returned string is != NULL and stays valid during @plugin's lifetime.
- */
-const gchar *
+ * Returns: textual name of @plugin. If the real name is not available
+ * 	for some reason, automatically generated string will be returned.
+ * 	Returned string is != NULL and stays valid during @plugin's lifetime.
+ **/
+gchar const *
 go_plugin_get_name (GOPlugin *plugin)
 {
 	g_return_val_if_fail (IS_GO_PLUGIN (plugin), NULL);
@@ -483,13 +486,12 @@ go_plugin_get_name (GOPlugin *plugin)
 
 /**
  * go_plugin_get_description:
- * @plugin      : The plugin
+ * @plugin : #GOPlugin
  *
- * Returns textual description of @plugin or NULL if description is not
- * available.
- * Returned string stays valid during @plugin's lifetime.
- */
-const gchar *
+ * Returns: textual description of @plugin or NULL if description is not
+ * 	available.  Returned string stays valid during @plugin's lifetime.
+ **/
+gchar const *
 go_plugin_get_description (GOPlugin *plugin)
 {
 	g_return_val_if_fail (IS_GO_PLUGIN (plugin), NULL);
@@ -502,10 +504,10 @@ go_plugin_get_description (GOPlugin *plugin)
 
 /**
  * go_plugin_is_loaded:
- * @plugin      : The plugin
+ * @plugin : #GOPlugin
  *
- * Returns : TRUE if @plugin is loaded and FALSE otherwise.
- */
+ * Returns: TRUE if @plugin is loaded and FALSE otherwise.
+ **/
 gboolean
 go_plugin_is_loaded (GOPlugin *plugin)
 {
@@ -526,10 +528,11 @@ go_plugin_is_loaded (GOPlugin *plugin)
  * consists of loader's plugin id and service id concatenated using colon).
  * All requests to create new loader object of this type will be passed to
  * @service.
+ *
  * This function is intended for use by GOPluginService objects.
- */
+ **/
 void
-go_plugins_register_loader (const gchar *loader_id, GOPluginService *service)
+go_plugins_register_loader (gchar const *loader_id, GOPluginService *service)
 {
 	g_return_if_fail (loader_id != NULL);
 	g_return_if_fail (service != NULL);
@@ -544,10 +547,11 @@ go_plugins_register_loader (const gchar *loader_id, GOPluginService *service)
  * Unregisters a type of plugin loader identified by @loader_id. After
  * callingthis function Gnumeric will be unable to load plugins supported
  * by the specified loader.
+ *
  * This function is intended for use by GOPluginService objects.
- */
+ **/
 void
-go_plugins_unregister_loader (const gchar *loader_id)
+go_plugins_unregister_loader (gchar const *loader_id)
 {
 	g_return_if_fail (loader_id != NULL);
 
@@ -555,7 +559,7 @@ go_plugins_unregister_loader (const gchar *loader_id)
 }
 
 static GType
-get_loader_type_by_id (const gchar *id_str, ErrorInfo **ret_error)
+get_loader_type_by_id (gchar const *id_str, ErrorInfo **ret_error)
 {
 	GOPluginService *loader_service;
 	ErrorInfo *error;
@@ -715,7 +719,7 @@ plugin_dependency_free (gpointer data)
 }
 
 static void
-go_plugin_read (GOPlugin *plugin, const gchar *dir_name, ErrorInfo **ret_error)
+go_plugin_read (GOPlugin *plugin, gchar const *dir_name, ErrorInfo **ret_error)
 {
 	gchar *file_name;
 	xmlDocPtr doc;
@@ -733,7 +737,8 @@ go_plugin_read (GOPlugin *plugin, const gchar *dir_name, ErrorInfo **ret_error)
 	file_name = g_build_filename (dir_name, PLUGIN_INFO_FILE_NAME, NULL);
 	doc = go_xml_parse_file (file_name);
 	if (doc == NULL || doc->xmlRootNode == NULL || strcmp (doc->xmlRootNode->name, "plugin") != 0) {
-		if (access (file_name, R_OK) != 0) {
+		char *uri = go_filename_to_uri (file_name);
+		if (go_file_access (uri, R_OK) != 0) {
 			*ret_error = error_info_new_printf (
 			             _("Can't read plugin info file (\"%s\")."),
 			             file_name);
@@ -743,6 +748,7 @@ go_plugin_read (GOPlugin *plugin, const gchar *dir_name, ErrorInfo **ret_error)
 			             file_name);
 		}
 		g_free (file_name);
+		g_free (uri);
 		xmlFreeDoc (doc);
 		return;
 	}
@@ -867,7 +873,7 @@ static void
 plugin_get_loader_if_needed (GOPlugin *plugin, ErrorInfo **ret_error)
 {
 	GType loader_type;
-	ErrorInfo *error;
+	ErrorInfo *error = NULL;
 
 	g_return_if_fail (IS_GO_PLUGIN (plugin));
 
@@ -901,7 +907,7 @@ plugin_get_loader_if_needed (GOPlugin *plugin, ErrorInfo **ret_error)
 
 /**
  * go_plugin_activate:
- * @plugin      : The plugin
+ * @plugin : #GOPlugin
  * @ret_error   : Pointer used to report errors
  *
  * Activates @plugin together with all its dependencies.
@@ -991,7 +997,7 @@ go_plugin_activate (GOPlugin *plugin, ErrorInfo **ret_error)
 
 /**
  * go_plugin_deactivate:
- * @plugin      : The plugin
+ * @plugin : #GOPlugin
  * @ret_error   : Pointer used to report errors
  *
  * Dectivates @plugin. Its dependencies will NOT be automatically
@@ -1050,11 +1056,11 @@ go_plugin_deactivate (GOPlugin *plugin, ErrorInfo **ret_error)
 
 /**
  * go_plugin_can_deactivate:
- * @plugin       : The plugin
+ * @plugin : #GOPlugin
  *
  * Tells if the plugin can be deactivated using go_plugin_deactivate.
  *
- * Returns : TRUE if @plugin can be deactivated and FALSE otherwise.
+ * Returns: TRUE if @plugin can be deactivated and FALSE otherwise.
  */
 gboolean
 go_plugin_can_deactivate (GOPlugin *plugin)
@@ -1148,7 +1154,7 @@ go_plugin_load_base (GOPlugin *plugin, ErrorInfo **ret_error)
 
 /**
  * go_plugin_load_service:
- * @plugin       : The plugin
+ * @plugin : #GOPlugin
  * @service     : Plugin service
  * @ret_error   : Pointer used to report errors
  *
@@ -1172,7 +1178,7 @@ go_plugin_load_service (GOPlugin *plugin, GOPluginService *service, ErrorInfo **
 
 /**
  * go_plugin_unload_service:
- * @plugin       : The plugin
+ * @plugin : #GOPlugin
  * @service     : Plugin service
  * @ret_error   : Pointer used to report errors
  *
@@ -1195,7 +1201,7 @@ go_plugin_unload_service (GOPlugin *plugin, GOPluginService *service, ErrorInfo
 
 /**
  * go_plugin_use_ref:
- * @plugin      : The plugin
+ * @plugin : #GOPlugin
  */
 void
 go_plugin_use_ref (GOPlugin *plugin)
@@ -1211,8 +1217,8 @@ go_plugin_use_ref (GOPlugin *plugin)
 
 /**
  * go_plugin_use_unref:
- * @plugin      : The plugin
- */
+ * @plugin : #GOPlugin
+ **/
 void
 go_plugin_use_unref (GOPlugin *plugin)
 {
@@ -1228,14 +1234,14 @@ go_plugin_use_unref (GOPlugin *plugin)
 
 /**
  * go_plugin_get_dependencies_ids:
- * @plugin      : The plugin
+ * @plugin : #GOPlugin
  *
- * Returns the list of identifiers of plugins that @plugin depends on.
- * All these plugins will be automatically activated before activating
- * the @plugin itself.
- * The caller must free the returned list together with the strings it
- * points to (use go_slist_free_custom (list, g_free) to do this).
- */
+ * Returns: the list of identifiers of plugins that @plugin depends on.
+ * 	All these plugins will be automatically activated before activating the
+ * 	@plugin itself.  The caller must free the returned list together with
+ * 	the strings it points to (use go_slist_free_custom (list, g_free) to do
+ * 	this).
+ **/
 GSList *
 go_plugin_get_dependencies_ids (GOPlugin *plugin)
 {
@@ -1250,9 +1256,10 @@ go_plugin_get_dependencies_ids (GOPlugin *plugin)
 
 /**
  * go_plugin_get_services:
- * @plugin      : The plugin
+ * @plugin : #GOPlugin
  *
- */
+ * Returns: A list of services.  The list must not be freed or changed.
+ **/
 GSList *
 go_plugin_get_services (GOPlugin *plugin)
 {
@@ -1265,7 +1272,7 @@ go_plugin_get_services (GOPlugin *plugin)
  * May return NULL without errors (is XML file doesn't exist)
  */
 static GOPlugin *
-go_plugin_read_for_dir (const gchar *dir_name, ErrorInfo **ret_error)
+go_plugin_read_for_dir (gchar const *dir_name, ErrorInfo **ret_error)
 {
 	GOPlugin *plugin = NULL;
 	gchar *file_name;
@@ -1323,7 +1330,7 @@ go_plugin_read_for_dir (const gchar *dir_name, ErrorInfo **ret_error)
  * May return partial list and some error info.
  */
 static GSList *
-go_plugin_list_read_for_subdirs_of_dir (const gchar *dir_name, ErrorInfo **ret_error)
+go_plugin_list_read_for_subdirs_of_dir (gchar const *dir_name, ErrorInfo **ret_error)
 {
 	GSList *plugins = NULL;
 	GDir *dir;
@@ -1339,7 +1346,7 @@ go_plugin_list_read_for_subdirs_of_dir (const gchar *dir_name, ErrorInfo **ret_e
 
 	while ((d_name = g_dir_read_name (dir)) != NULL) {
 		gchar *full_entry_name;
-		ErrorInfo *error;
+		ErrorInfo *error = NULL;
 		GOPlugin *plugin;
 
 		if (strcmp (d_name, ".") == 0 || strcmp (d_name, "..") == 0)
@@ -1376,7 +1383,7 @@ go_plugin_list_read_for_subdirs_of_dir_list (GSList *dir_list, ErrorInfo **ret_e
 	GO_INIT_RET_ERROR_INFO (ret_error);
 	for (dir_iterator = dir_list; dir_iterator != NULL; dir_iterator = dir_iterator->next) {
 		gchar *dir_name;
-		ErrorInfo *error;
+		ErrorInfo *error = NULL;
 		GSList *dir_plugin_info_list;
 
 		dir_name = (gchar *) dir_iterator->data;
@@ -1478,8 +1485,8 @@ go_plugin_db_deactivate_plugin_list (GSList *plugins, ErrorInfo **ret_error)
 /**
  * plugins_get_available_plugins:
  *
- * Returns the list of available plugins. The returned value must not be
- * freed and stays valid until calling plugins_rescan or plugins_shutdown.
+ * Returns: the list of available plugins. The returned value must not be freed
+ * 	and stays valid until calling plugins_rescan or plugins_shutdown.
  **/
 GSList *
 go_plugins_get_available_plugins (void)
@@ -1490,8 +1497,8 @@ go_plugins_get_available_plugins (void)
 /**
  * plugins_get_active_plugins:
  *
- * Returns the list of active plugins.  The caller needs to free the list, but
- * not the content.
+ * Returns: the list of active plugins.  The caller needs to free the list, but
+ * 	not the content.
  **/
 GSList *
 go_plugins_get_active_plugins (void)
@@ -1511,13 +1518,13 @@ go_plugins_get_active_plugins (void)
  * plugins_get_plugin_by_id:
  * @plugin_id    : String containing plugin ID
  *
- * Returns GOPlugin object for plugin with ID equal to @plugin_id or NULL
- * if there's no plugin available with given id.
- * Function returns "borrowed" reference, use g_object_ref if you want to
- * be sure that plugin won't disappear.
+ * Returns: GOPlugin object for plugin with ID equal to @plugin_id or NULL
+ * 	if there's no plugin available with given id.  Function returns
+ * 	"borrowed" reference, use g_object_ref if you want to be sure that
+ * 	plugin won't disappear.
  */
 GOPlugin *
-go_plugins_get_plugin_by_id (const gchar *plugin_id)
+go_plugins_get_plugin_by_id (gchar const *plugin_id)
 {
 	g_return_val_if_fail (plugin_id != NULL, NULL);
 
@@ -1526,7 +1533,7 @@ go_plugins_get_plugin_by_id (const gchar *plugin_id)
 
 /**
  * plugin_db_mark_plugin_for_deactivation :
- * @plugin:
+ * @plugin : #GOPlugin
  * @mark:
  */
 void
@@ -1548,7 +1555,7 @@ go_plugin_db_mark_plugin_for_deactivation (GOPlugin *plugin, gboolean mark)
 
 /**
  * plugin_db_is_plugin_marked_for_deactivation :
- * @plugin: 
+ * @plugin : #GOPlugin
  */
 gboolean
 go_plugin_db_is_plugin_marked_for_deactivation (GOPlugin *plugin)
@@ -1690,10 +1697,15 @@ go_plugins_set_dirs (GSList *plugin_dirs)
 /**
  * go_plugins_init:
  * @context     : #GOCmdContext used to report errors
+ * @known_states : A list of known states (defined how ?)
+ * @active_plugins: A list of active plugins
+ * @plugin_dirs :a list of directories to search for plugins
+ * @activate_new_plugins : activate plugins we have no seen before.
+ * @default_loader_type : importer to use by default.
  *
  * Initializes the plugin subsystem. Don't call this function more than
  * once.
- */
+ **/
 void
 go_plugins_init (GOCmdContext *context,
 		 GSList const *known_states,
@@ -1772,6 +1784,88 @@ go_plugins_init (GOCmdContext *context,
 	}
 }
 
+/**
+ * go_plugins_add:
+ * @context     : #GOCmdContext used to report errors
+ * @known_states : A list of known states (defined how ?)
+ * @active_plugins: A list of active plugins
+ * @plugin_dirs :a list of directories to search for plugins
+ * @default_loader_type : importer to use by default.
+ *
+ * Adds new plugins to currently used plugins.
+ **/
+void
+go_plugins_add (GOCmdContext *context,
+		GSList const *known_states,
+		GSList const *active_plugins,
+		GSList *plugin_dirs,
+		GType  default_loader_type)
+{
+	GSList *error_list = NULL;
+	ErrorInfo *error;
+	GSList *plugin_list;
+
+	plugin_dirs = g_slist_append (plugin_dirs, go_plugins_get_plugin_dir ());
+
+	go_default_loader_type = default_loader_type;
+	go_plugins_set_dirs (plugin_dirs);
+
+	GO_SLIST_FOREACH (known_states, char, state_str,
+		PluginFileState *state;
+
+		state = plugin_file_state_from_string (state_str);
+		if (state != NULL)
+			g_hash_table_insert (plugin_file_state_dir_hash, state->dir_name, state);
+	);
+	plugin_file_state_hash_changed = FALSE;
+
+	/* collect information about the available plugins */
+	g_slist_free (available_plugins);
+	available_plugins = go_plugin_list_read_for_all_dirs (&error);
+	GO_SLIST_FOREACH (available_plugins, GOPlugin, plugin,
+		g_hash_table_insert (
+			available_plugins_id_hash,
+			(gpointer) go_plugin_get_id (plugin), plugin);
+	);
+	if (error != NULL) {
+		GO_SLIST_PREPEND (error_list, error_info_new_str_with_details (
+			_("Errors while reading info about available plugins."), error));
+	}
+
+	/* get descriptors for all previously active plugins */
+	plugin_list = NULL;
+	GO_SLIST_FOREACH (active_plugins, char, plugin_id,
+		GOPlugin *plugin = go_plugins_get_plugin_by_id (plugin_id);
+		if (plugin != NULL)
+			GO_SLIST_PREPEND (plugin_list, plugin);
+	);
+
+	/* get descriptors for new plugins */
+	g_hash_table_foreach (
+		plugin_file_state_dir_hash,
+		(GHFunc) ghf_collect_new_plugins,
+		&plugin_list);
+
+	plugin_list = g_slist_reverse (plugin_list);
+	go_plugin_db_activate_plugin_list (plugin_list, &error);
+	g_slist_free (plugin_list);
+	if (error != NULL) {
+		GO_SLIST_PREPEND (error_list, error_info_new_str_with_details (
+			_("Errors while activating plugins."), error));
+	}
+
+	/* report initialization errors */
+	if (error_list != NULL) {
+		GO_SLIST_REVERSE (error_list);
+		error = error_info_new_str_with_details_list (
+		        _("Errors while initializing plugin system."),
+		        error_list);
+
+		go_cmd_context_error_info (context, error);
+		error_info_free (error);
+	}
+}
+
 static void
 ghf_collect_used_plugin_state_strings (gpointer key, gpointer value, gpointer user_data)
 {
