@@ -102,8 +102,7 @@ char const *GOPriorityName[] = {
 static void
 go_mime_type_free (GOMimeType * mime_type)
 {
-	if (mime_type->component_type_name)
-		g_free (mime_type->component_type_name);
+	g_free (mime_type->component_type_name);
 	g_free (mime_type);
 }
 
@@ -122,10 +121,10 @@ go_component_type_service_read_xml (GOPluginService * service, xmlNode * tree,
 					       (GDestroyNotify)
 					       go_mime_type_free);
 	for (ptr = tree->xmlChildrenNode; ptr != NULL; ptr = ptr->next)
-		if (0 == xmlStrcmp (ptr->name, "mime_type"))
-		{
+		if (0 == xmlStrcmp (ptr->name, "mime_type")) {
 			char *name = xmlGetProp (ptr, "name");
 			char *priority = xmlGetProp (ptr, "priority");
+			char *support_clipboard = xmlGetProp (ptr, "clipboard");
 			GOMimeType *mime_type =
 				g_hash_table_lookup (mime_types, name);
 			int i;
@@ -133,7 +132,7 @@ go_component_type_service_read_xml (GOPluginService * service, xmlNode * tree,
 			for (i = 4; i >= 0; i--)
 				if (!strcmp (priority, GOPriorityName[i]))
 					break;
-			g_free (priority);
+			xmlFree (priority);
 
 /* FIXME FIXME FIXME the code should take into account that a plugin might be deactivated ! */
 			if (mime_type == NULL) {
@@ -141,18 +140,22 @@ go_component_type_service_read_xml (GOPluginService * service, xmlNode * tree,
 				mime_type->priority = i;
 				mime_type->component_type_name =
 					g_strdup (service->id);
+				mime_type->support_clipboard = (support_clipboard &&
+					!strcmp (support_clipboard, "yes"))? TRUE: FALSE;
 				mime_types_names =
 					g_slist_append (mime_types_names,
 							name);
 				g_hash_table_replace (mime_types, name,
 						      mime_type);
 			} else if (i > mime_type->priority) {
-				g_free (name);
+				xmlFree (name);
 				g_free (mime_type->component_type_name);
 				mime_type->component_type_name =
 					g_strdup (service->id);
 				mime_type->priority = i;
 			}
+			if (support_clipboard)
+				xmlFree (support_clipboard);
 		}
 }
 
@@ -225,7 +228,7 @@ goc_plugin_services_shutdown (void)
 }
 
 GSList *
-go_components_get_mime_types ()
+go_components_get_mime_types (void)
 {
 	return mime_types_names;
 }
@@ -237,6 +240,13 @@ go_components_get_priority (char const *mime_type)
 	return (t) ? t->priority : GO_MIME_PRIORITY_INVALID;
 }
 
+gboolean
+go_components_support_clipboard (char const *mime_type)
+{
+	GOMimeType *t = g_hash_table_lookup (mime_types, mime_type);
+	return (t) ? t->support_clipboard : FALSE;
+}
+
 void
 go_components_add_mime_type (char *mime, GOMimePriority priority, char const *service_id)
 {
