@@ -45,11 +45,13 @@ typedef struct _GogSeriesDimDesc GogSeriesDimDesc; /* dimension of a series */
 typedef struct _GogLegend	GogLegend;
 typedef struct _GogLabel	GogLabel;
 typedef struct _GogStyledObject	GogStyledObject;
+typedef struct _GogAxisBase GogAxisBase;
 typedef struct _GogAxis		GogAxis;
 typedef struct _GogAxisLine	GogAxisLine;
 typedef struct _GogGrid		GogGrid;
 typedef struct _GogGridLine	GogGridLine;
 typedef struct _GogErrorBar	GogErrorBar;
+typedef struct _GogEquation	GogEquation;
 typedef struct _GogRegCurve	GogRegCurve;
 typedef struct _GogRegEqn	GogRegEqn;
 typedef struct _GogTrendLineType	GogTrendLineType;
