@@ -35,33 +35,27 @@ typedef struct  {
 		int size_pts;
 } GogAxisTickProperties;
 
-typedef struct {
+struct _GogAxisBase {
 	GogStyledObject	 base;
 
 	GogChart	*chart;
 	GogAxis		*axis;
 
-	GogAxisPosition    pos_user;		/* user assigned preference */
-	GogAxisPosition    pos_calculated;	/* actual position, with current AxisSet */
+	GogAxisPosition    position;
 	unsigned 	   crossed_axis_id;
 	GogDatasetElement  cross_location;
-	
+
 	int		   padding;
-	
+
 	GogAxisTickProperties major, minor;
 	gboolean major_tick_labeled;
-} GogAxisBase;
+};
 
 typedef GogStyledObjectClass GogAxisBaseClass;
 
-#define GOG_AXIS_BASE_TYPE	(gog_axis_base_get_type ())
-#define GOG_AXIS_BASE(o)	(G_TYPE_CHECK_INSTANCE_CAST ((o), GOG_AXIS_BASE_TYPE, GogAxisBase))
-#define IS_GOG_AXIS_BASE(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), GOG_AXIS_BASE_TYPE))
-
-GType gog_axis_base_get_type (void);
-
-GogAxisPosition 	gog_axis_base_get_position (GogAxisBase *axis_base);
-void		    	gog_axis_base_set_position (GogAxisBase *axis_base, GogAxisPosition position);
+GogAxisPosition 	gog_axis_base_get_position 	   (GogAxisBase *axis_base);
+void		    	gog_axis_base_set_position 	   (GogAxisBase *axis_base, GogAxisPosition position);
+GogAxisPosition 	gog_axis_base_get_clamped_position (GogAxisBase *axis_base);
 
 typedef struct {
 	GogView		base;
@@ -77,6 +71,9 @@ typedef GogViewClass	GogAxisBaseViewClass;
 #define IS_GOG_AXIS_BASE_VIEW(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), GOG_AXIS_BASE_VIEW_TYPE))
 
 GType gog_axis_base_view_get_type (void);
+void gog_axis_base_view_label_position_request (GogView *view,
+			                        GogViewAllocation const *bbox,
+						GogViewAllocation *pos);
 
 G_END_DECLS
 
