@@ -88,6 +88,7 @@ struct _GogSeries {
 	GList		  *overrides;  /* GogSeriesElement (individual points) */
 
 	GOLineInterpolation	interpolation;
+	gboolean interpolation_skip_invalid;
 };
 
 typedef struct {
@@ -104,6 +105,7 @@ typedef struct {
 	void (*dim_changed) (GogSeries *series, int dim_i);
 	unsigned (*get_xy_data) (GogSeries const *series,
 					double const **x, double const **y);
+	GogDataset *(*get_interpolation_params) (GogSeries const *series);
 } GogSeriesClass;
 
 #define GOG_SERIES_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST ((k), GOG_SERIES_TYPE, GogSeriesClass))
