@@ -33,7 +33,7 @@
 #include <glib-object.h>
 #include <goffice/gtk/go-color-group.h>
 #include <goffice/utils/go-color.h>
-#include <gtk/gtkwidget.h>
+#include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
@@ -53,8 +53,6 @@ GOColor go_combo_color_get_color (GOComboColor  *cc, gboolean *is_default);
 void    go_combo_color_set_color (GOComboColor  *cc, GOColor   color);
 void    go_combo_color_set_color_to_default (GOComboColor *cc);
 void    go_combo_color_set_color_gdk (GOComboColor *cc, GdkColor *color);
-GOColor go_combo_color_get_default   (GOComboColor *cc);
-void	go_combo_color_set_default   (GOComboColor *cc, GOColor color);
 
 void go_combo_color_set_allow_alpha    (GOComboColor *cc, gboolean allow_alpha);
 void go_combo_color_set_instant_apply  (GOComboColor *cc, gboolean active);
