@@ -14,14 +14,6 @@
 #include <goffice/gtk/goffice-gtk.h>
 #include <goffice/utils/go-color.h>
 
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkentry.h>
-
 #include <goffice/cut-n-paste/foocanvas/foo-canvas.h>
 #include <goffice/cut-n-paste/foocanvas/foo-canvas-text.h>
 
@@ -357,8 +349,9 @@ gfs_init (GOFontSel *gfs)
 
 	gfs->modifications = pango_attr_list_new ();
 
-	gtk_box_pack_start_defaults (GTK_BOX (gfs),
-		glade_xml_get_widget (gfs->gui, "toplevel-table"));
+	gtk_box_pack_start (GTK_BOX (gfs), 
+		glade_xml_get_widget (gfs->gui, "toplevel-table"), 
+		TRUE, TRUE, 0);
 	gfs->font_name_entry  = glade_xml_get_widget (gfs->gui, "font-name-entry");
 	gfs->font_style_entry = glade_xml_get_widget (gfs->gui, "font-style-entry");
 	gfs->font_size_entry  = glade_xml_get_widget (gfs->gui, "font-size-entry");
