@@ -779,11 +779,32 @@ typedef struct {
 	char const *link;
 } CBHelpPaths;
 
-#ifdef GOFFICE_WITH_GNOME
-#include <libgnome/gnome-help.h>
-#elif defined(G_OS_WIN32)
+#ifdef G_OS_WIN32
 #include <windows.h>
-#include <htmlhelp.h>
+
+typedef HWND (* WINAPI HtmlHelpA_t) (HWND hwndCaller,
+				     LPCSTR pszFile,
+				     UINT uCommand,
+				     DWORD_PTR dwData);
+#define HH_HELP_CONTEXT 0x000F
+
+static HtmlHelpA_t
+html_help (void)
+{
+	HMODULE hhctrl;
+	static HtmlHelpA_t result = NULL;
+	static gboolean beenhere = FALSE;
+
+	if (!beenhere) {
+		hhctrl = LoadLibrary ("hhctrl.ocx");
+		if (hhctrl != NULL)
+			result = (HtmlHelpA_t) GetProcAddress (hhctrl, "HtmlHelpA");
+		beenhere = TRUE;
+	}
+
+	return result;
+}
+
 #endif
 static void
 go_help_display (CBHelpPaths const *paths)
@@ -837,19 +858,16 @@ go_help_display (CBHelpPaths const *paths)
 		gchar *path = g_build_filename (paths->data_dir, "doc", "C", chmfile, NULL);
 
 		g_free (chmfile);
-		if (!HtmlHelp (GetDesktopWindow (), path, HH_HELP_CONTEXT, id))
-			go_gtk_notice_dialog (NULL, GTK_MESSAGE_ERROR, "Failed to spawn HtmlHelp");
+		if (html_help () == NULL ||
+		    !(html_help ()) (GetDesktopWindow (), path, HH_HELP_CONTEXT, id))
+			go_gtk_notice_dialog (NULL, GTK_MESSAGE_ERROR, "Failed to load HtmlHelp");
 		g_free (path);
 	}
 #else
-#ifdef GOFFICE_WITH_GNOME
-	if (gnome_program_get () != NULL) {
-		gnome_help_display (paths->app, paths->link, NULL);
-		return;
-	}
-#endif
-	go_gtk_notice_dialog (NULL, GTK_MESSAGE_ERROR,
-			      "TODO : launch help browser for %s", paths->link);
+	char *uri = g_strconcat ("ghelp:", paths->app, "#", paths->link, NULL);
+	go_url_show (uri);
+	g_free (uri);
+	return;
 #endif
 }
 
@@ -924,6 +942,8 @@ go_gtk_url_is_writeable (GtkWindow *parent, char const *uri,
 			_("A file called <i>%s</i> already exists in %s.\n\n"
 			  "Do you want to save over it?"),
 			basename, dirname);
+		/* gcc complains over this, but the format seems right and
+		   is not supposed to be constant.  Using "%s" is wrong.  */
 		GtkWidget *dialog = gtk_message_dialog_new_with_markup (parent,
 			 GTK_DIALOG_DESTROY_WITH_PARENT,
 			 GTK_MESSAGE_WARNING,
