@@ -19,9 +19,8 @@
 #ifndef _GOFFICE_GTK_H_
 #define _GOFFICE_GTK_H_
 
-#include <gtk/gtkmessagedialog.h>
+#include <gtk/gtk.h>
 #include <glade/glade-xml.h>
-#include <gtk/gtkfilechooser.h>
 #include <goffice/app/goffice-app.h>
 #include <goffice/utils/go-image.h>
 
@@ -29,7 +28,7 @@ G_BEGIN_DECLS
 
 void	   go_gtk_editable_enters (GtkWindow *window, GtkWidget *w);
 
-/* go_libglade_new deprecated in 0.4.3, will be removed in 0.5.x */
+/* go_libglade_new deprecated in 0.4.3, will be removed in 0.7.x */
 #define go_libglade_new	go_glade_new
 GladeXML  *go_glade_new		   	   (char const *gladefile, char const *root,
 					    char const *domain, GOCmdContext *cc);
@@ -44,14 +43,6 @@ gulong	   go_glade_signal_connect_swapped (GladeXML *gui,
 					    GCallback	 c_handler,
 					    gpointer	 data);
 
-GdkPixbuf *go_pixbuf_new_from_file	(char const *filename);
-GdkPixbuf *go_pixbuf_intelligent_scale	(GdkPixbuf *pixbuf, 
-					 guint width, guint height);
-void       go_cairo_set_source_pixbuf (cairo_t   *cr,
-				       GdkPixbuf *pixbuf,
-				       double     pixbuf_x,
-				       double     pixbuf_y);
-
 int	   go_pango_measure_string	(PangoContext *context,
 					 PangoFontDescription const *font_desc,
 					 char const *str);
@@ -85,6 +76,12 @@ void	   go_atk_setup_label	 	(GtkWidget *label, GtkWidget *target);
 
 void       go_dialog_guess_alternative_button_order (GtkDialog *dialog);
 
+void       go_widget_set_tooltip_text (GtkWidget *widget, const gchar *tip);
+void       go_tool_item_set_tooltip_text (GtkToolItem *item, const gchar *tip);
+
+void 	   go_menu_position_below 		(GtkMenu *menu, gint *x, gint *y,
+						 gint *push_in, gpointer user_data);
+
 G_END_DECLS
 
 #endif /* _GOFFICE_GTK_H_ */
