@@ -27,14 +27,14 @@
 #include <goffice/ms-compat/god-image-ms.h>
 #include <goffice/drawing/god-property-table.h>
 #include <goffice/drawing/god-shape.h>
-#include <glib/gmacros.h>
+#include <glib.h>
 #include <gsf/gsf-input.h>
 #include <gsf/gsf-utils.h>
 #include <string.h>
 
 #define d(x) x
 
-#define CVS_VERSION "$Id: god-drawing-ms.c 1164 2005-08-08 08:57:07Z jdassen $"
+#define CVS_VERSION "$Id: god-drawing-ms.c 2295 2008-12-14 21:58:11Z mortenw $"
 #define ERROR_STRING(cond,str) G_STRLOC "\n<" CVS_VERSION ">\n" str " (" #cond ")"
 #define ERROR(cond,str) { \
 	if (!(cond)) { \
